﻿using Connection.Class.GATT;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;

namespace WBP_Office_API_Demo_Customer {

    public partial class Form_BLE_Discover : Form {
        public string BLE_ID { get; set; }
        public BLEAction BLEAction { get; set; }

        private delegate void DoActionHandler(ListBox lb, Action action);

        public Form_BLE_Discover() {
            InitializeComponent();
            Connection.BLE_Scanner.InitScan(new List<string>(new string[] { "WatchBP Office" }));
            Connection.BLE_Scanner.DeviceAdded += OnDeviceAdded;
            Connection.BLE_Scanner.DeviceRemoved += OnDeviceRemoved;
            Connection.BLE_Scanner.SignalStrengthUpdated += DeviceWatcher_SignalStrengthUpdated;
            Connection.BLE_Scanner.StartScan();
        }

        private void DeviceWatcher_SignalStrengthUpdated(object sender, Dictionary<ulong, int> e) {
        }

        private void OnDeviceRemoved(object sender, DeviceRemovedEventArgs e) {
            UpdateUI(listBox1, () => {
                for (int i = 0; i < listBox1.Items.Count; i++) {
                    if (listBox1.Items[i].ToString().Contains(e.Device.Id)) {
                        listBox1.Items.Remove(listBox1.Items[i]);
                        return;
                    }
                }
            });
        }

        private void OnDeviceAdded(object sender, DeviceAddedEventArgs e) {
            UpdateUI(listBox1, () => listBox1.Items.Add($"{(e.Device.IsPaired ? "!" : "@")}{e.Device.Name} -- {e.Device.Address:X} -- {e.Device.Id}"));
        }

        public static void UpdateUI(ListBox tb, Action action) {
            //判斷這個TextBox的物件是否在同一個執行緒上
            if (tb.InvokeRequired) {
                //當InvokeRequired為true時，表示在不同的執行緒上，所以進行委派的動作!!
                DoActionHandler ph = new DoActionHandler(UpdateUI);
                tb.Invoke(ph, tb, action);
            } else {
                //表示在同一個執行緒上了，所以可以正常的呼叫到這個TextBox物件
                action.Invoke();
            }
        }

        private void Button1_Click(object sender, EventArgs e) {
            if (listBox1.SelectedItem == null)
                return;
            BLE_ID = listBox1.SelectedItem.ToString().Split(new string[] { " -- " }, StringSplitOptions.RemoveEmptyEntries).Last();
            BLEAction = BLEAction.Connect;
            Close();
        }

        private void Button2_Click(object sender, EventArgs e) {
            BLEAction = BLEAction.Disconnect;
            Close();
        }

        private void Button3_Click(object sender, EventArgs e) {
            if (listBox1.SelectedItem == null)
                return;
            BLE_ID = listBox1.SelectedItem.ToString().Split(new string[] { " -- " }, StringSplitOptions.RemoveEmptyEntries).Last();
            BLEAction = BLEAction.Pair;
            Close();
        }

        private void Button4_Click(object sender, EventArgs e) {
            if (listBox1.SelectedItem == null)
                return;
            BLE_ID = listBox1.SelectedItem.ToString().Split(new string[] { " -- " }, StringSplitOptions.RemoveEmptyEntries).Last();
            BLEAction = BLEAction.Unpair;
            Close();
        }
    }

    public enum BLEAction {
        None,
        Connect,
        Disconnect,
        Pair,
        Unpair
    }
}